package de.intevation.flys.artifacts.states;

import java.util.Collection;
import java.util.Map;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifactdatabase.ProtocolUtils;
import de.intevation.artifactdatabase.data.StateData;
import de.intevation.artifactdatabase.state.AbstractState;

import de.intevation.flys.artifacts.resources.Resources;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public abstract class DefaultState extends AbstractState {

    /** The logger that is used in this class.*/
    private static Logger logger = Logger.getLogger(DefaultState.class);


    public Element describeStatic(
        Document    document,
        Node        root,
        CallContext context,
        String      uuid)
    {
        XMLUtils.ElementCreator creator = new XMLUtils.ElementCreator(
            document,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        CallMeta meta = context.getMeta();

        String label = Resources.getMsg(meta, getID(), getID());
        Element ui   = ProtocolUtils.createArtNode(
            creator, "state",
            new String[] { "name", "uiprovider", "label" },
            new String[] { getID(), getUIProvider(), label });

        Map<String, StateData> theData = getData();
        if (theData == null) {
            return ui;
        }

        Collection<StateData> dataItems = theData.values();

        for (StateData data: dataItems) {
            String name  = data.getName();
            String value = (String) data.getValue();

            logger.debug("Append element '" + name + "' (" + value + ")");

            Element dataElement = creator.create("data");
            creator.addAttr(dataElement, "name", name, true);
            creator.addAttr(dataElement, "type", data.getType(), true);

            Element itemElement = creator.create("item");
            creator.addAttr(itemElement, "value", value, true);
            creator.addAttr(
                itemElement,
                "label",
                Resources.getMsg(meta, value, value),
                true);

            dataElement.appendChild(itemElement);
            ui.appendChild(dataElement);
        }

        return ui;
    }


    public Element describe(
        Artifact    artifact,
        Document    document,
        Node        root,
        CallContext context,
        String      uuid)
    {
        XMLUtils.ElementCreator creator = new XMLUtils.ElementCreator(
            document,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        Element ui        = null;
        String uiprovider = getUIProvider();
        if (uiprovider != null) {
            ui  = ProtocolUtils.createArtNode(
                creator, "dynamic",
                new String[] { "uiprovider" },
                new String[] { uiprovider });
        }
        else {
            ui  = ProtocolUtils.createArtNode(creator, "dynamic", null, null);
        }

        Map<String, StateData> theData = getData();
        if (theData == null) {
            return ui;
        }

        Collection<StateData> dataItems = theData.values();

        for (StateData data: dataItems) {
            String name    = data.getName();
            Element select = createData(creator, artifact, data, context);

            Element choices = ProtocolUtils.createArtNode(
            creator, "choices", null, null);

            select.appendChild(choices);
            ui.appendChild(select);

            Element[] items = createItems(creator, artifact, name, context);
            for (Element item: items) {
                choices.appendChild(item);
            }
        }

        return ui;
    }


    /**
     * This method creates the root node that contains the list of selectable
     * items.
     *
     * @param cr The ElementCreator.
     * @param name The name of the amount of data.
     *
     * @return the root node of the item list.
     */
    protected Element createData(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        StateData   data,
        CallContext context)
    {
        Element select = ProtocolUtils.createArtNode(
            cr, "select", null, null);
        cr.addAttr(select, "name", data.getName(), true);

        Element label = ProtocolUtils.createArtNode(
            cr, "label", null, null);

        select.appendChild(label);

        label.setTextContent(Resources.getMsg(
            context.getMeta(),
            getID(),
            getID()));

        return select;
    }


    /**
     * This method creates a list of items. These items represent the amount of
     * input data that is possible for this state.
     *
     * @param cr The ElementCreator.
     * @param name The name of the amount of data.
     *
     * @return a list of items.
     */
    protected abstract Element[] createItems(
        XMLUtils.ElementCreator cr,
        Artifact    artifact,
        String      name,
        CallContext context);


    protected String getUIProvider() {
        return null;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
